package com.loafle.overflow.container.service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

import com.loafle.overflow.core.annotation.RPCService;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.model.sensorconfig.SensorConfig;
import com.loafle.overflow.service.container.CollectorService;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * ContainerCollectorService
 */
@RPCService("CollectorService")
public class ContainerCollectorService implements CollectorService {
  private static final Logger logger = LoggerFactory.getLogger(ContainerCollectorService.class);

  private ScheduledExecutorService scheduledExecutorService;
  private Map<String, ScheduledFuture<?>> scheduleMap;

  @Autowired
  private ContainerProbeService probeService;
  @Autowired
  private ContainerCrawlerService crawlerService;

  @Override
  public void initService() throws Exception {
    this.scheduledExecutorService = Executors.newScheduledThreadPool(20);
    this.scheduleMap = new HashMap<>();
  }

  @Override
  public void startService() throws Exception {
  }

  @Override
  public void stopService() {
    this.scheduledExecutorService.shutdown();
  }

  @Override
  public void destroyService() {

  }

  public void addSensorConfigs(List<SensorConfig> sensorConfigs) throws OverflowException {
    if (null == sensorConfigs || 0 == sensorConfigs.size()) {
      return;
    }

    for (SensorConfig sensorConfig : sensorConfigs) {
      try {
        long interval = Long.valueOf(sensorConfig.getSchedule().getInterval()).longValue();
        this.addSchedule(interval, sensorConfig);
        logger.debug("scheduler of config[%s] has been added", sensorConfig.getConfigID());
      } catch (Exception e) {
        throw new OverflowException(String.format("Cannot convert interval[%s] %v", sensorConfig.getSchedule().getInterval()));
      }
    }
  }

  public void removeSensorConfigs(List<SensorConfig> sensorConfigs) throws OverflowException {
    if (null == sensorConfigs || 0 == sensorConfigs.size()) {
      return;
    }

    for (SensorConfig sensorConfig : sensorConfigs) {
      this.removeSchedule(sensorConfig.getConfigID());
    }
  }

  private void addSchedule(long interval, SensorConfig sensorConfig) {
    Runnable task = () -> {
      try {
        Map<String, String> result = crawlerService.get(sensorConfig.getConfigID());
        probeService.send("DataService.Metric", result);
      } catch (Exception e) {
        logger.error("Cannot get data from crawler[%s] %s", sensorConfig.getCrawler().getName(), e.getMessage());
      }
    };
    ScheduledFuture<?> sf = this.scheduledExecutorService.schedule(task, interval, TimeUnit.SECONDS);
    this.scheduleMap.put(sensorConfig.getConfigID(), sf);
  }

  private void removeSchedule(String configID) {
    if (!this.scheduleMap.containsKey(configID)) {
      return;
    }
    ScheduledFuture<?> sf = this.scheduleMap.get(configID);
    this.scheduleMap.remove(configID);
    sf.cancel(false);
  }
}